#!/usr/bin/env python
# -*- coding: UTF-8 -*-
#
import os
from   time     import strftime
from   sqlite3  import dbapi2 as sqlite
from   .mumamisc import *

SELECTCOUNT = "SELECT Count(*) "


class MusicManDb(object):
    TAB_TITLE = 'muma_title'
    TAB_PLACE = 'muma_place'
    TAB_UPDATE = 'muma_update'

    def __init__(self, filename = 'musicman.db3'):
        self.filename = filename
        self.connectOrCreate()
    #-------------------------------------------------------------------------------------------

    def connectOrCreate(self):        
        self.connection = sqlite.connect(self.filename, isolation_level = None)
        if sys.version < '3':
            self.connection.text_factory = lambda x: unicode(x, "utf-8", "replace")
        else:
            self.connection.text_factory = lambda x: str(x, "utf-8", "replace")

        self.cursor = self.connection.cursor()
        self.transaction = False
        ismuma, tabcount = self.checkDatabase()
        if tabcount < 2:        
            try:
                self.cursor.execute('CREATE TABLE ' + self.TAB_TITLE +
                                    ''' (
                                         Id_Title    integer primary key,
                                         Title       varchar(50) collate nocase,
                                         Interpreter varchar(45) collate nocase,
                                         Duration    time,
                                         Date        date,
                                         Code        varchar(7) collate nocase,
                                         Composer    varchar(40) collate nocase
                                        )''')
                self.cursor.execute('CREATE INDEX TitleInterp ON ' + self.TAB_TITLE + '(Title, Interpreter collate nocase)')             
                self.cursor.execute('CREATE TABLE ' + self.TAB_PLACE +
                                    ''' (
                                         Id_Place    integer primary key autoincrement,
                                         Title_Id    integer key,
                                         Pdate       date,
                                         Place       integer
                                        )''')
                self.cursor.execute('PRAGMA default_cache_size=32000')
            except:
                pass
    #-------------------------------------------------------------------------------------------

    def checkDatabase(self):
        self.commitTransaction()
        self.cursor.execute("SELECT name FROM sqlite_master WHERE type='table'")
        record = self.cursor.fetchall()
        tabnames = []
        for item in record:
            tabnames.append(item[0])
        if not self.TAB_UPDATE in tabnames:
            self.cursor.execute('CREATE TABLE ' + self.TAB_UPDATE +
                                ''' (
                                     LastUpdate      date
                                    )''')
            self.cursor.execute("INSERT INTO %s VALUES ('%s')" % (self.TAB_UPDATE, strftime('%Y-%m-%d %H:%M:%S')))            

        return self.TAB_TITLE in tabnames and self.TAB_PLACE in tabnames, len(tabnames)
    #-------------------------------------------------------------------------------------------

    def compactDatabase(self):
        self.commitTransaction()
        self.cursor.execute("VACUUM")
    #-------------------------------------------------------------------------------------------

    def titleExists(self, id_title):
        self.cursor.execute('SELECT Id_Title FROM %s WHERE Id_Title = %s' % (self.TAB_TITLE, id_title))
        try:
            retval = self.cursor.fetchone()[0] > 0
        except:
            retval = False
        return retval
    #-------------------------------------------------------------------------------------------
                
    def patchTables(self, pstarget, pssource, delentry):
        if delentry:
            self.deleteRecordById(pstarget)
        self.cursor.execute("UPDATE %s SET Title_Id = %s WHERE Title_Id = %s" % (self.TAB_PLACE, pssource, pstarget))
    #-------------------------------------------------------------------------------------------

    def deleteRecordById(self, recordid):
        self.commitTransaction()
        self.cursor.execute("DELETE FROM %s WHERE Id_Title = %s" % (self.TAB_TITLE, recordid))
    #-------------------------------------------------------------------------------------------

    def isUniqueId(self, recordid):
        self.commitTransaction()
        try:
            self.cursor.execute("SELECT Id_Title FROM %s WHERE Id_Title = %s" % (self.TAB_TITLE, recordid))
            retval = self.cursor.fetchone()[0] == None
        except:
            retval = True
        return retval
    #-------------------------------------------------------------------------------------------

    def getInterTitleById(self, recordid):
        self.commitTransaction()
        try:
            self.cursor.execute("SELECT Title, Interpreter FROM %s WHERE Id_Title = %s" % (self.TAB_TITLE, recordid))
            result = self.cursor.fetchone()
            retval = "%s-%s" % (result[1], result[0])
            retval = retval.replace(' ', '_')
            retval = retval.replace('*', '')
            retval = retval.replace('?', '')
            retval = retval.replace(':', '_')
            if LINUX:
                retval = retval.replace('/', '!')
            else:
                retval = retval.replace('\\', '!')
        except:
            retval = ''
        return retval
    #-------------------------------------------------------------------------------------------

    def getTitleById(self, recordid):
        self.commitTransaction()
        try:
            self.cursor.execute("SELECT Title FROM %s WHERE Id_Title = %s" % (self.TAB_TITLE, recordid))
            retval = self.cursor.fetchone()[0]
        except:
            retval = ''
        return retval
    #-------------------------------------------------------------------------------------------

    def getDurationById(self, recordid):
        self.commitTransaction()
        try:
            self.cursor.execute("SELECT Duration FROM %s WHERE Id_Title = %s" % (self.TAB_TITLE, recordid))
            retval = self.cursor.fetchone()[0]
        except:
            retval = '00:00:00'
        return retval
    #-------------------------------------------------------------------------------------------

    def getRecordCount(self, tablename):
        self.commitTransaction()
        try:
            self.cursor.execute('%s FROM "%s"' % (SELECTCOUNT, tablename))
            return self.cursor.fetchone()[0]
        except:
            return 0
    #-------------------------------------------------------------------------------------------

    def getLastUpdate(self):
        self.commitTransaction()
        try:
            self.cursor.execute('SELECT LastUpdate FROM %s' % (self.TAB_UPDATE))
            return self.cursor.fetchone()[0]
        except:
            return '2000-01-01  00:00:00'
    #-------------------------------------------------------------------------------------------

    def checkSqlSyntax(self, query):
        """   """
        self.commitTransaction()
        try:
            self.cursor.execute('EXPLAIN ' + query)
            result = self.cursor.fetchall()
            return None
        #except sqlite.Error, err:      # (Python 2)
        except sqlite.Error as err:
            return err
    #-------------------------------------------------------------------------------------------

    def execQuery(self, query):
        """   """
        self.commitTransaction()
        self.cursor.execute(query)
    #-------------------------------------------------------------------------------------------
    
    def getMatches(self, query):
        """   """
        self.commitTransaction()
        self.cursor.execute(query)
        result = self.cursor.fetchall()
        try:
            return result, len(result)
        except:
            return None, 0
    #-------------------------------------------------------------------------------------------
    
    def putTitleRecord(self, record):
        if not self.transaction:
            self.cursor.execute('BEGIN TRANSACTION')
            self.transaction = True
        query = 'INSERT INTO %s VALUES ("%s", "%s", "%s", "%s", "%s", "%s", "%s")' % \
                                       (self.TAB_TITLE, record[0], record[1], record[2],
                                        record[3], record[4], record[5], record[6])
        self.cursor.execute(query)
    #-------------------------------------------------------------------------------------------
    
    def putPlaceRecord(self, record):
        if not self.transaction:
            self.cursor.execute('BEGIN TRANSACTION')
            self.transaction = True
    
        query = 'INSERT INTO %s VALUES (NULL, "%s", "%s", "%s")' % (self.TAB_PLACE, \
                                        record[0], record[1], record[2])
        self.cursor.execute(query)
    #-------------------------------------------------------------------------------------------
    
    def commitTransaction(self):
        if self.transaction:
            self.cursor.execute('UPDATE %s SET LastUpdate = "%s"' % (self.TAB_UPDATE, strftime('%Y-%m-%d  %H:%M:%S')))
            self.cursor.execute('COMMIT TRANSACTION')
            self.transaction = False
    #-------------------------------------------------------------------------------------------

    def closeAll(self):
        self.commitTransaction()
        self.cursor.close()
        self.connection.close()
    #-------------------------------------------------------------------------------------------

# test main
if (__name__ == '__main__'):
    mumadb = MusicManDb()
    
    infile = file('Tabelle.csv', 'rt')
    lines = infile.readlines()
    infile.close()
    for line in lines:
        if line.endswith('\n'):
            line = line.replace('\n', '')
        try:
            splitline = line.split(';')
            # del splitline[3]      # das ist der Playlink gewesen
            if len(splitline[4]):                
                date = splitline[4].split('.')
                if date[2] < '40':
                    splitline[4] = '20%s-%s-%s' % (date[2], date[1], date[0])
                else:
                    splitline[4] = '19%s-%s-%s' % (date[2], date[1], date[0])
            for i in range(6):
                splitline[i] = splitline[i].strip()
            record = tuple(splitline)
            #try:
            mumadb.putTitleRecord(record)
            #except:
             #   print 'Crash: keine einmalige Id'
        except:
            print(line)
            
    mumadb.commitTransaction()
    
    infile = file('Platzierung.csv', 'rt')
    lines = infile.readlines()
    infile.close()
    for line in lines:
        splitline = line.split(';')
        date = splitline[1].split('.')
        if date[2] < '50':
            splitline[1] = '20%s-%s-%s' % (date[2], date[1], date[0])
        else:
            splitline[1] = '19%s-%s-%s' % (date[2], date[1], date[0])
        for i in range(3):
            splitline[i] = splitline[i].strip()
        record = tuple(splitline)        
        mumadb.putPlaceRecord(record)
        
    mumadb.commitTransaction()
    
    print 'Titel - Records =', mumadb.getRecordCount(mumadb.TAB_TITLE)
    print 'Platz - Records =', mumadb.getRecordCount(mumadb.TAB_PLACE)
    mumadb.closeAll()